@@ -40,6 +40,7 @@
 #include <libgnomevfs/gnome-vfs.h>
 
 #include <libgnome/gnome-desktop-item.h>
+#include <menu-tree.h>
 
 #include <bonobo/bonobo-exception.h>
 #include <bonobo-activation/bonobo-activation.h>
@@ -947,11 +948,10 @@ load_bugzillas (void)
 	d(g_list_foreach (druid_data.dldests, (GFunc)p_string, NULL));
 }
 
-#define ALL_APPLICATIONS_URI "applications-all-users:///"
 #define APPLET_REQUIREMENTS \
 	"has_all (repo_ids, ['IDL:Bonobo/Control:1.0'," \
 	"		     'IDL:GNOME/Vertigo/PanelAppletShell:1.0']) && " \
-	"defined (panel:icon) && defined (panel:category)"
+	"defined (panel:icon)"
 
 #define BUGZILLA_BUGZILLA               "X-GNOME-Bugzilla-Bugzilla"
 #define BUGZILLA_PRODUCT                "X-GNOME-Bugzilla-Product"
@@ -980,9 +980,6 @@ bugzilla_application_new (const char *name,
 			  const char *other_programs)
 {
 	BugzillaApplication *app;
-	GdkPixbuf *pb = NULL;
-	char *icon_copy;
-	GError *error = NULL;
 	char **programv;
 	int i;
 
@@ -996,27 +993,34 @@ bugzilla_application_new (const char *name,
 	app->version     = g_strdup (version);
 	app->email       = g_strdup (email);
 
-	if (icon && *icon != G_DIR_SEPARATOR)
-		icon_copy = gnome_program_locate_file (NULL,
-						       GNOME_FILE_DOMAIN_PIXMAP,
-						       icon, TRUE, NULL);
-	else
-		icon_copy = g_strdup (icon);
-
-	if (icon_copy) {
-		pb = gdk_pixbuf_new_from_file (icon_copy, &error);
-		g_free (icon_copy);
+	if (icon) {
+		if (g_path_is_absolute (icon)) {
+			app->pixbuf = gdk_pixbuf_new_from_file_at_size (icon, 24, 24, NULL);
+		}
+		if (app->pixbuf == NULL) {
+			app->pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+								icon, 24, 0, NULL);
+		}
+		if (app->pixbuf == NULL && strrchr (icon, '.') != NULL) {
+			char *name;
+			name = g_strndup (icon, strlen (icon) - strlen (strrchr (icon, '.')));
+			app->pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+								name, 24, 0, NULL);
+			g_free (name);
+		}
+	
+		if (app->pixbuf == NULL) {
+			char *filename;
+			filename = g_strdup_printf (BUDDY_ICONDIR"/pixmaps/%s", icon);
+			app->pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 24, 24, NULL);
+			g_free (filename);
+		}
 	}
 
-	if (pb) {
-		app->pixbuf = gdk_pixbuf_scale_simple (pb, 20, 20, GDK_INTERP_HYPER);
-		g_object_unref (pb);
-	} else if (error) {
-		g_warning (_("Couldn't load icon for %s: %s"), app->name, error->message);
-		g_error_free (error);
+	if (app->pixbuf == NULL) {
+		g_warning (_("Couldn't load icon for %s"), app->name);
 	}
 	
-
 	if (program) {
 		g_shell_parse_argv (program, &i, &programv, NULL);
 		if (programv[0]) {
@@ -1101,6 +1105,7 @@ NULL)) {
 			continue;
 		}
 
+
 		bugzilla_application_new (
 			bonobo_server_info_prop_lookup (info, "name", langs),
 			bonobo_server_info_prop_lookup (info, "description", langs),
@@ -1117,83 +1122,120 @@ NULL)) {
 	CORBA_free (info_list);
 }
 
-static gboolean
-visit_cb (const char *rel_path,
-	  GnomeVFSFileInfo *info,
-	  gboolean recursing_will_loop,
-	  gpointer data,
-	  gboolean *recurse)
+static int
+compare_applications (MenuTreeEntry *a,
+		      MenuTreeEntry *b)
 {
-	GnomeDesktopItem *ditem;
-	GError *error = NULL;
-	char *full_path, *icon;
-	GnomeIconTheme *git = data;
-	
-	if (info->name[0] != '.'
-	    || (info->name[1] != '.' && info->name[1] != 0)
-	    || info->name[2] != 0) {
-		if (recursing_will_loop) {
-			g_print ("Loop detected\n");
-			return TRUE;
-		}
-		*recurse = TRUE;
-	} else {
-		*recurse = FALSE;
-	}
-
-	full_path = g_strconcat (ALL_APPLICATIONS_URI, rel_path, NULL);
-
-	ditem = gnome_desktop_item_new_from_uri (full_path, 0, &error);
-	if (!ditem) {
-		if (error) {
-			g_warning ("Couldn't load %s: %s", full_path, error->message);
-		}
-		goto visit_cb_out;
-	}
-    
-	if ((gnome_desktop_item_get_entry_type (ditem) != GNOME_DESKTOP_ITEM_TYPE_APPLICATION) || !gnome_desktop_item_get_string (ditem, BUGZILLA_BUGZILLA))
-		goto visit_cb_out;
-
-	icon = gnome_desktop_item_get_icon (ditem, git);
-	bugzilla_application_new (
-		gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME),
-		gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_COMMENT),
-		gnome_desktop_item_get_string (ditem, BUGZILLA_BUGZILLA),
-		gnome_desktop_item_get_string (ditem, BUGZILLA_PRODUCT),
-		gnome_desktop_item_get_string (ditem, BUGZILLA_COMPONENT),
-		gnome_desktop_item_get_string (ditem, BUGZILLA_VERSION),
-		gnome_desktop_item_get_string (ditem, BUGZILLA_EMAIL),
-		icon,
-		gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC),
-		gnome_desktop_item_get_string (ditem, BUGZILLA_OTHER_BINARIES));
-	g_free (icon);
-
- visit_cb_out:
-	g_free (full_path);
-	if (error)
-		g_error_free (error);
-	if (ditem)
-		gnome_desktop_item_unref (ditem);
-
-	return TRUE;
+	return g_utf8_collate (menu_tree_entry_get_name (a),
+			       menu_tree_entry_get_name (b));
 }
 
+static GSList *
+get_all_applications_from_dir (MenuTreeDirectory *directory,
+			       GSList            *list)
+{
+	GSList *subdirs;
+	GSList *l;
+
+	list = g_slist_concat (list,
+			       menu_tree_directory_get_entries (directory));
+
+	subdirs = menu_tree_directory_get_subdirs (directory);
+	for (l = subdirs; l; l = l->next) {
+		MenuTreeDirectory *subdir = l->data;
+
+		list = get_all_applications_from_dir (subdir, list);
+
+		menu_tree_directory_unref (subdir);
+	}
+	g_slist_free (subdirs);
+
+	return list;
+}
+
+static GSList *
+get_all_applications (void)
+{
+	MenuTree          *tree;
+	MenuTreeDirectory *root;
+	GSList            *retval;
+
+	tree = menu_tree_lookup ("applications.menu");
+
+	root = menu_tree_get_root_directory (tree);
+
+	retval = get_all_applications_from_dir (root, NULL);
+
+	menu_tree_directory_unref (root);
+	menu_tree_unref (tree);
+
+	retval = g_slist_sort (retval, (GCompareFunc) compare_applications);
+
+	return retval;
+}
+
+
 void
 load_applications (void)
 {
 	GnomeIconTheme *git;
+	GSList         *all_applications;
+	GSList         *l;
+	const char     *prev_name;
+	GError	       *error = NULL;
 
 	druid_data.program_to_application = g_hash_table_new (g_str_hash, g_str_equal);
 
 	git = gnome_icon_theme_new ();
 	gnome_icon_theme_set_allow_svg (git, FALSE);
-	
-	/* load applications */
-	gnome_vfs_directory_visit (ALL_APPLICATIONS_URI,
-				   GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
-				   GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK,
-				   visit_cb, git);
 
+	all_applications = get_all_applications ();
+	for (l = all_applications; l; l = l->next) {
+		GnomeDesktopItem *ditem;
+		char *icon;
+		MenuTreeEntry *entry = l->data;
+
+		if (prev_name && strcmp (menu_tree_entry_get_name (entry), prev_name) == 0) {
+			menu_tree_entry_unref (entry);
+			continue;
+		}
+		ditem = gnome_desktop_item_new_from_uri (menu_tree_entry_get_desktop_file_path (entry), 
+							 0, &error);
+		if (!ditem) {
+			if (error) {
+				g_warning ("Couldn't load %s: %s", desktop_entry_get_path (entry),
+					   error->message);
+				g_error_free (error);
+				error = NULL;
+			}
+			menu_tree_entry_unref (entry);
+			continue;
+		}
+    
+		if ((gnome_desktop_item_get_entry_type (ditem) != GNOME_DESKTOP_ITEM_TYPE_APPLICATION) || !gnome_desktop_item_get_string (ditem, BUGZILLA_BUGZILLA)) {
+			gnome_desktop_item_unref (ditem);	
+			menu_tree_entry_unref (entry);
+			continue;
+		}
+
+		icon = gnome_desktop_item_get_icon (ditem, git);
+		bugzilla_application_new (
+			gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME),
+			gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_COMMENT),
+			gnome_desktop_item_get_string (ditem, BUGZILLA_BUGZILLA),
+			gnome_desktop_item_get_string (ditem, BUGZILLA_PRODUCT),
+			gnome_desktop_item_get_string (ditem, BUGZILLA_COMPONENT),
+			gnome_desktop_item_get_string (ditem, BUGZILLA_VERSION),
+			gnome_desktop_item_get_string (ditem, BUGZILLA_EMAIL),
+			icon,
+			gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC),
+			gnome_desktop_item_get_string (ditem, BUGZILLA_OTHER_BINARIES));
+		g_free (icon);
+		prev_name = menu_tree_entry_get_name (entry);
+		gnome_desktop_item_unref (ditem);	
+		menu_tree_entry_unref (entry);
+	}
+	g_slist_free (all_applications);
 
 	load_applets (git);
 
